@@ -6,9 +6,19 @@
 
     An out-of-the-box REST Web API that's as dangerous as you want it to be.
 
-    :copyright: (c) 2014 by Nicola Iarocci.
+    :copyright: (c) 2015 by Nicola Iarocci.
     :license: BSD, see LICENSE for more details.
 
+    .. versionchanged:: 0.5
+       'SERVER_NAME' removed.
+       'QUERY_WHERE' added.
+       'QUERY_SORT' added.
+       'QUERY_PAGE' added.
+       'QUERY_MAX_RESULTS' added.
+       'QUERY_PROJECTION' added.
+       'QUERY_EMBEDDED' added.
+       'RFC1123_DATE_FORMAT' added.
+
     .. versionchanged:: 0.4
        'META' defaults to '_meta'.
        'ERROR' defaults to '_error'.
@@ -28,14 +38,14 @@
 
 """
 
-__version__ = '0.4'
+__version__ = '0.5'
 
 # RFC 1123 (ex RFC 822)
 DATE_FORMAT = '%a, %d %b %Y %H:%M:%S GMT'
+RFC1123_DATE_FORMAT = '%a, %d %b %Y %H:%M:%S GMT'
 
 URL_PREFIX = ''
 API_VERSION = ''
-SERVER_NAME = None
 PAGINATION = True
 PAGINATION_LIMIT = 50
 PAGINATION_DEFAULT = 25
@@ -62,5 +72,14 @@ ETAG = '_etag'
 VERSION = '_version'
 META = '_meta'
 
+QUERY_WHERE = 'where'
+QUERY_SORT = 'sort'
+QUERY_PAGE = 'page'
+QUERY_MAX_RESULTS = 'max_results'
+QUERY_EMBEDDED = 'embedded'
+QUERY_PROJECTION = 'projection'
+
+VALIDATION_ERROR_STATUS = 422
+
 # must be the last line (will raise W402 on pyflakes)
 from eve.flaskapp import Eve  # noqa
