@@ -7,7 +7,7 @@
     This module implements the central WSGI application object as a Flask
     subclass.
 
-    :copyright: (c) 2014 by Nicola Iarocci.
+    :copyright: (c) 2015 by Nicola Iarocci.
     :license: BSD, see LICENSE for more details.
 """
 
@@ -145,12 +145,22 @@ class Eve(Flask, Events):
         else:
             self.auth = None
 
-        # set up home url
         self._init_url_rules()
 
+        self._init_oplog()
+
         # validate and set defaults for each resource
-        for resource, settings in self.config['DOMAIN'].items():
+
+        # Use a snapshot of the DOMAIN setup for iteration so
+        # further insertion of versioned resources do not
+        # cause a RuntimeError due to the change of size of
+        # the dict
+        domain_copy = copy.deepcopy(self.config['DOMAIN'])
+        for resource, settings in domain_copy.items():
             self.register_resource(resource, settings)
+        # it seems like both domain_copy and config['DOMAIN']
+        # suffered changes at this point, so merge them
+        # self.config['DOMAIN'].update(domain_copy)
 
         self.register_error_handlers()
 
@@ -182,6 +192,9 @@ class Eve(Flask, Events):
         Since we are a Flask subclass, any configuration value supported by
         Flask itself is available (besides Eve's proper settings).
 
+        .. versionchanged:: 0.5
+           Allow EVE_SETTINGS envvar to be used exclusively. Closes #461.
+
         .. versionchanged:: 0.2
            Allow use of a dict object as settings.
         """
@@ -198,7 +211,11 @@ class Eve(Flask, Events):
             else:
                 abspath = os.path.abspath(os.path.dirname(sys.argv[0]))
                 pyfile = os.path.join(abspath, self.settings)
-            self.config.from_pyfile(pyfile)
+            try:
+                self.config.from_pyfile(pyfile)
+            except:
+                # assume an envvar is going to be used exclusively
+                pass
 
         # overwrite settings with custom environment variable
         envvar = 'EVE_SETTINGS'
@@ -326,6 +343,9 @@ class Eve(Flask, Events):
         :param resource: resource name.
         :param schema: schema definition for the resource.
 
+        .. versionchanged:: 0.5
+           Add ETAG to automatic fields check.
+
         .. versionchanged:: 0.4
            Checks against offending document versioning fields.
            Supports embedded data_relation with version.
@@ -346,7 +366,7 @@ class Eve(Flask, Events):
            the exception message.
         """
         # ensure automatically handled fields aren't defined
-        fields = [eve.DATE_CREATED, eve.LAST_UPDATED]
+        fields = [eve.DATE_CREATED, eve.LAST_UPDATED, eve.ETAG]
         # TODO: only add the following checks if settings['versioning'] == True
         fields += [
             self.config['VERSION'],
@@ -371,9 +391,6 @@ class Eve(Flask, Events):
                     raise SchemaException("'resource' key is mandatory for "
                                           "the 'data_relation' rule in "
                                           "'%s: %s'" % (resource, field))
-                # If the field is listed as `embeddable`
-                # it must be type == 'objectid'
-                # TODO: allow serializing a list( type == 'objectid')
                 if ruleset['data_relation'].get('embeddable', False):
 
                     # special care for data_relations with a version
@@ -387,16 +404,6 @@ class Eve(Flask, Events):
                                 "declaring an embedded data_relation with"
                                 " version." % value_field
                             )
-                        else:
-                            type = ruleset['schema'][value_field]['type']
-                    else:
-                        type = ruleset['type']
-
-                    if type != 'objectid':
-                        raise SchemaException(
-                            "In order for the 'data_relation' rule to be "
-                            "embeddable it must be of type 'objectid'"
-                        )
 
         # TODO are there other mandatory settings? Validate them here
 
@@ -461,6 +468,10 @@ class Eve(Flask, Events):
     def _set_resource_defaults(self, resource, settings):
         """ Low-level method which sets default values for one resource.
 
+        .. versionchanged:: 0.5
+           Don't set default projection if 'allow_unknown' is active (#497).
+           'internal_resource'
+
         .. versionchanged:: 0.3
            Set projection to None when schema is not provided for the resource.
            Support for '_media' helper.
@@ -507,6 +518,8 @@ class Eve(Flask, Events):
         settings.setdefault('pagination', self.config['PAGINATION'])
         settings.setdefault('projection', self.config['PROJECTION'])
         settings.setdefault('versioning', self.config['VERSIONING'])
+        settings.setdefault('internal_resource',
+                            self.config['INTERNAL_RESOURCE'])
         # TODO make sure that this we really need the test below
         if settings['item_lookup']:
             item_methods = self.config['ITEM_METHODS']
@@ -533,7 +546,7 @@ class Eve(Flask, Events):
         settings['datasource'].setdefault('filter', None)
         settings['datasource'].setdefault('default_sort', None)
 
-        if len(schema):
+        if len(schema) and settings['allow_unknown'] is False:
             # enable retrieval of actual schema fields only. Eventual db
             # fields not included in the schema won't be returned.
             projection = {}
@@ -541,6 +554,7 @@ class Eve(Flask, Events):
             projection[self.config['ID_FIELD']] = 1
             projection[self.config['LAST_UPDATED']] = 1
             projection[self.config['DATE_CREATED']] = 1
+            projection[self.config['ETAG']] = 1
             if settings['versioning'] is True:
                 projection[self.config['VERSION']] = 1
                 projection[
@@ -603,12 +617,25 @@ class Eve(Flask, Events):
         """ Builds the API url map for one resource. Methods are enabled for
         each mapped endpoint, as configured in the settings.
 
+        .. versionchanged:: 0.5
+           Don't add resource to url rules if it's flagged as internal.
+           Strip regexes out of config.URLS helper. Closes #466.
+
         .. versionadded:: 0.2
         """
-        url = '%s/%s' % (self.api_prefix, settings['url'])
-        self.config['URLS'][resource] = settings['url']
         self.config['SOURCES'][resource] = settings['datasource']
 
+        if settings['internal_resource']:
+            return
+
+        url = '%s/%s' % (self.api_prefix, settings['url'])
+
+        pretty_url = settings['url']
+        if '<' in pretty_url:
+            pretty_url = pretty_url[:pretty_url.index('<')+1] + \
+                pretty_url[pretty_url.index(':')+1:]
+        self.config['URLS'][resource] = pretty_url
+
         # resource endpoint
         endpoint = resource + "|resource"
         self.add_url_rule(url, endpoint, view_func=collections_endpoint,
@@ -725,6 +752,11 @@ class Eve(Flask, Events):
                 copy.deepcopy(self.config['SOURCES'][resource])
             self.config['SOURCES'][versioned_resource]['source'] += \
                 self.config['VERSIONS']
+            # the new versioned resource also needs URL rules
+            self._add_resource_url_rules(
+                versioned_resource,
+                self.config['DOMAIN'][versioned_resource]
+            )
 
     def register_error_handlers(self):
         """ Register custom error handlers so we make sure that all errors
@@ -732,5 +764,42 @@ class Eve(Flask, Events):
 
         .. versionadded:: 0.4
         """
-        for code in [400, 401, 403, 404]:
+        for code in [400, 401, 403, 404, 405, 406, 409, 410, 422]:
             self.error_handler_spec[None][code] = error_endpoint
+
+    def _init_oplog(self):
+        """ If enabled, configures the OPLOG endpoint.
+
+        .. versionadded:: 0.5
+        """
+        name, endpoint, audit = (
+            self.config['OPLOG_NAME'],
+            self.config['OPLOG_ENDPOINT'],
+            self.config['OPLOG_AUDIT']
+        )
+
+        if endpoint:
+            settings = self.config['DOMAIN'].setdefault(name, {})
+
+            settings.setdefault('url', endpoint)
+            settings.setdefault('datasource', {'source': name})
+
+            # this endpoint is always read-only
+            settings['resource_methods'] = ['GET']
+            settings['item_methods'] = ['GET']
+
+            # schema is also fixed. it is needed because otherwise we
+            # would end up exposing the AUTH_FIELD when User-Restricted-
+            # Resource-Access is enabled.
+            settings['schema'] = {
+                'r': {},
+                'o': {},
+                'i': {},
+            }
+            if audit:
+                settings['schema'].update(
+                    {
+                        'ip': {},
+                        'c': {}
+                    }
+                )
