@@ -7,11 +7,12 @@
     This module implements a Python-to-Mongo syntax parser. Allows the MongoDB
     data-layer to seamlessy respond to a Python-like query.
 
-    :copyright: (c) 2014 by Nicola Iarocci.
+    :copyright: (c) 2015 by Nicola Iarocci.
     :license: BSD, see LICENSE for more details.
 """
 
 import ast
+import sys
 from datetime import datetime   # noqa
 from bson import ObjectId       # noqa
 
@@ -23,7 +24,12 @@ def parse(expression):
     supported.
     """
     v = MongoVisitor()
-    v.visit(ast.parse(expression))
+    try:
+        v.visit(ast.parse(expression))
+    except SyntaxError as e:
+        e = ParseError(e)
+        e.__traceback__ = sys.exc_info()[2]
+        raise e
     return v.mongo_query
 
 
