@@ -6,7 +6,7 @@
 
     This module imlements the PATCH method.
 
-    :copyright: (c) 2014 by Nicola Iarocci.
+    :copyright: (c) 2015 by Nicola Iarocci.
     :license: BSD, see LICENSE for more details.
 """
 
@@ -18,7 +18,8 @@ from eve.auth import requires_auth
 from eve.validation import ValidationError
 from eve.methods.common import get_document, parse, payload as payload_, \
     ratelimit, pre_event, store_media_files, resolve_embedded_fields, \
-    build_response_document, marshal_write_response
+    build_response_document, marshal_write_response, resolve_document_etag, \
+    oplog_push
 from eve.versioning import resolve_document_version, \
     insert_versioning_documents, late_versioning_catch
 
@@ -26,15 +27,52 @@ from eve.versioning import resolve_document_version, \
 @ratelimit()
 @requires_auth('item')
 @pre_event
-def patch(resource, **lookup):
-    """ Perform a document patch/update. Updates are first validated against
-    the resource schema. If validation passes, the document is updated and
-    an OK status update is returned. If validation fails, a set of validation
-    issues is returned.
+def patch(resource, payload=None, **lookup):
+    """
+    Default function for handling PATCH requests, it has decorators for
+    rate limiting, authentication and for raising pre-request events.
+    After the decorators are applied forwards to call to :func:`patch_internal`
+
+    .. versionchanged:: 0.5
+       Split into patch() and patch_internal().
+    """
+    return patch_internal(resource, payload, concurrency_check=True,
+                          skip_validation=False, **lookup)
+
+
+def patch_internal(resource, payload=None, concurrency_check=False,
+                   skip_validation=False, **lookup):
+    """ Intended for internal patch calls, this method is not rate limited,
+    authentication is not checked, pre-request events are not raised, and
+    concurrency checking is optional. Performs a document patch/update.
+    Updates are first validated against the resource schema. If validation
+    passes, the document is updated and an OK status update is returned.
+    If validation fails, a set of validation issues is returned.
 
     :param resource: the name of the resource to which the document belongs.
+    :param payload: alternative payload. When calling patch() from your own
+                    code you can provide an alternative payload. This can be
+                    useful, for example, when you have a callback function
+                    hooked to a certain endpoint, and want to perform
+                    additional patch() callsfrom there.
+
+                    Please be advised that in order to successfully use this
+                    option, a request context must be available.
+    :param concurrency_check: concurrency check switch (bool)
+    :param skip_validation: skip payload validation before write (bool)
     :param **lookup: document lookup query.
 
+    .. versionchanged:: 0.5
+       Updating nested document fields does not overwrite the nested document
+       itself (#519).
+       Push updates to the OpLog.
+       Original patch() has been split into patch() and patch_internal().
+       You can now pass a pre-defined custom payload to the funcion.
+       ETAG is now stored with the document (#369).
+       Catching all HTTPExceptions and returning them to the caller, allowing
+       for eventual flask.abort() invocations in callback functions to go
+       through. Fixes #395.
+
     .. versionchanged:: 0.4
        Allow abort() to be inoked by callback functions.
        'on_update' raised before performing the update on the database.
@@ -82,15 +120,18 @@ def patch(resource, **lookup):
     .. versionchanged:: 0.0.3
        JSON links. Superflous ``response`` container removed.
     """
-    payload = payload_()
-    original = get_document(resource, **lookup)
+    if payload is None:
+        payload = payload_()
+
+    original = get_document(resource, concurrency_check, **lookup)
     if not original:
         # not found
         abort(404)
 
     resource_def = app.config['DOMAIN'][resource]
     schema = resource_def['schema']
-    validator = app.validator(schema, resource)
+    if not skip_validation:
+        validator = app.validator(schema, resource)
 
     object_id = original[config.ID_FIELD]
     last_modified = None
@@ -107,7 +148,11 @@ def patch(resource, **lookup):
 
     try:
         updates = parse(payload, resource)
-        validation = validator.validate_update(updates, object_id)
+        if skip_validation:
+            validation = True
+        else:
+            validation = validator.validate_update(updates, object_id,
+                                                   original)
         if validation:
             # sneak in a shadow copy if it wasn't already there
             late_versioning_catch(original, resource)
@@ -130,9 +175,19 @@ def patch(resource, **lookup):
             getattr(app, "on_update")(resource, updates, original)
             getattr(app, "on_update_%s" % resource)(updates, original)
 
+            updates = resolve_nested_documents(updates, updated)
             updated.update(updates)
 
+            if config.IF_MATCH:
+                resolve_document_etag(updated)
+                # now storing the (updated) ETAG with every document (#453)
+                updates[config.ETAG] = updated[config.ETAG]
+
             app.data.update(resource, object_id, updates)
+
+            # update oplog if needed
+            oplog_push(resource, updates, 'PATCH', object_id)
+
             insert_versioning_documents(resource, updated)
 
             # nofity callbacks
@@ -150,8 +205,7 @@ def patch(resource, **lookup):
         # TODO should probably log the error and abort 400 instead (when we
         # got logging)
         issues['validator exception'] = str(e)
-    except (exceptions.InternalServerError, exceptions.Unauthorized,
-            exceptions.Forbidden, exceptions.NotFound) as e:
+    except exceptions.HTTPException as e:
         raise e
     except Exception as e:
         # consider all other exceptions as Bad Requests
@@ -162,10 +216,32 @@ def patch(resource, **lookup):
     if len(issues):
         response[config.ISSUES] = issues
         response[config.STATUS] = config.STATUS_ERR
+        status = config.VALIDATION_ERROR_STATUS
     else:
         response[config.STATUS] = config.STATUS_OK
+        status = 200
 
     # limit what actually gets sent to minimize bandwidth usage
     response = marshal_write_response(response, resource)
 
-    return response, last_modified, etag, 200
+    return response, last_modified, etag, status
+
+
+def resolve_nested_documents(updates, original):
+    """ Nested document updates are merged with the original contents
+    we don't overwrite the whole thing. See #519 for details.
+
+    .. versionadded:: 0.5
+    """
+    r = {}
+    for field, value in updates.items():
+        if isinstance(value, dict):
+            orig_value = original.setdefault(field, {})
+            if orig_value is None:
+                r[field] = value
+            else:
+                orig_value.update(resolve_nested_documents(value, orig_value))
+                r[field] = orig_value
+        else:
+            r[field] = value
+    return r
