@@ -93,6 +93,9 @@ class TestMinimal(unittest.TestCase):
     def assert201(self, status):
         self.assertEqual(status, 201)
 
+    def assert204(self, status):
+        self.assertEqual(status, 204)
+
     def assert301(self, status):
         self.assertEqual(status, 301)
 
@@ -102,6 +105,9 @@ class TestMinimal(unittest.TestCase):
     def assert304(self, status):
         self.assertEqual(status, 304)
 
+    def assert422(self, status):
+        self.assertEqual(status, 422)
+
     def get(self, resource, query='', item=None):
         if resource in self.domain:
             resource = self.domain[resource]['url']
@@ -145,6 +151,10 @@ class TestMinimal(unittest.TestCase):
             v = None
         return v, r.status_code
 
+    def assertValidationErrorStatus(self, status):
+        self.assertEqual(status,
+                         self.app.config.get('VALIDATION_ERROR_STATUS'))
+
     def assertValidationError(self, response, matches):
         self.assertTrue(eve.STATUS in response)
         self.assertTrue(eve.STATUS_ERR in response[eve.STATUS])
@@ -206,13 +216,15 @@ class TestMinimal(unittest.TestCase):
         self.assertItemLink(link, _id)
 
     def assertPagination(self, response, page, total, max_results):
+        p_key, mr_key = self.app.config['QUERY_PAGE'], \
+            self.app.config['QUERY_MAX_RESULTS']
         self.assertTrue(self.app.config['META'] in response)
         meta = response.get(self.app.config['META'])
-        self.assertTrue('page' in meta)
-        self.assertTrue('max_results' in meta)
+        self.assertTrue(p_key in meta)
+        self.assertTrue(mr_key in meta)
         self.assertTrue('total' in meta)
-        self.assertEqual(meta['page'], page)
-        self.assertEqual(meta['max_results'], max_results)
+        self.assertEqual(meta[p_key], page)
+        self.assertEqual(meta[mr_key], max_results)
         self.assertEqual(meta['total'], total)
 
     def assertHomeLink(self, links):
@@ -221,8 +233,7 @@ class TestMinimal(unittest.TestCase):
         self.assertTrue('title' in link)
         self.assertTrue('href' in link)
         self.assertEqual('home', link['title'])
-        self.assertEqual("%s" % self._get_server_name(),
-                         link['href'])
+        self.assertEqual("/", link['href'])
 
     def assertResourceLink(self, links, resource):
         self.assertTrue('self' in links)
@@ -231,9 +242,7 @@ class TestMinimal(unittest.TestCase):
         self.assertTrue('href' in link)
         url = self.domain[resource]['url']
         self.assertEqual(url, link['title'])
-        self.assertEqual("%s/%s" % (self._get_server_name(),
-                                    url),
-                         link['href'])
+        self.assertEqual("%s" % url, link['href'])
 
     def assertCollectionLink(self, links, resource):
         self.assertTrue('collection' in links)
@@ -242,8 +251,7 @@ class TestMinimal(unittest.TestCase):
         self.assertTrue('href' in link)
         url = self.domain[resource]['url']
         self.assertEqual(url, link['title'])
-        self.assertEqual("%s/%s" % (self._get_server_name(),
-                                    url), link['href'])
+        self.assertEqual("%s" % url, link['href'])
 
     def assertNextLink(self, links, page):
         self.assertTrue('next' in links)
@@ -251,7 +259,8 @@ class TestMinimal(unittest.TestCase):
         self.assertTrue('title' in link)
         self.assertTrue('href' in link)
         self.assertEqual('next page', link['title'])
-        self.assertTrue("page=%d" % page in link['href'])
+        self.assertTrue("%s=%d" % (self.app.config['QUERY_PAGE'], page)
+                        in link['href'])
 
     def assertPrevLink(self, links, page):
         self.assertTrue('prev' in links)
@@ -260,7 +269,8 @@ class TestMinimal(unittest.TestCase):
         self.assertTrue('href' in link)
         self.assertEqual('previous page', link['title'])
         if page > 1:
-            self.assertTrue("page=%d" % page in link['href'])
+            self.assertTrue("%s=%d" % (self.app.config['QUERY_PAGE'], page)
+                            in link['href'])
 
     def assertItemLink(self, links, item_id):
         self.assertTrue('self' in links)
@@ -277,7 +287,8 @@ class TestMinimal(unittest.TestCase):
             self.assertTrue('title' in link)
             self.assertTrue('href' in link)
             self.assertEqual('last page', link['title'])
-            self.assertTrue("page=%d" % page in link['href'])
+            self.assertTrue("%s=%d" % (self.app.config['QUERY_PAGE'], page)
+                            in link['href'])
         else:
             self.assertTrue('last' not in links)
 
@@ -313,13 +324,6 @@ class TestMinimal(unittest.TestCase):
     def bulk_insert(self):
         pass
 
-    def _get_server_name(self):
-        server_name = self.app.config.get('SERVER_NAME', '')
-        url_protocol = self.app.config.get('URL_PROTOCOL', '')
-        if url_protocol:
-            server_name = '%s://%s' % (url_protocol, server_name)
-        return server_name
-
     def dropDB(self):
         self.connection = MongoClient(MONGO_HOST, MONGO_PORT)
         self.connection.drop_database(MONGO_DBNAME)
@@ -489,10 +493,24 @@ class TestBase(TestMinimal):
             )
         return rows
 
+    def random_internal_transactions(self, num):
+        transactions = []
+        for i in range(num):
+            dt = datetime.now()
+            transaction = {
+                'internal_string':  self.random_string(10),
+                'internal_number': i,
+                eve.LAST_UPDATED: dt,
+                eve.DATE_CREATED: dt,
+            }
+            transactions.append(transaction)
+        return transactions
+
     def bulk_insert(self):
         _db = self.connection[MONGO_DBNAME]
         _db.contacts.insert(self.random_contacts(self.known_resource_count))
         _db.contacts.insert(self.random_users(2))
         _db.payments.insert(self.random_payments(10))
         _db.invoices.insert(self.random_invoices(1))
+        _db.internal_transactions.insert(self.random_internal_transactions(4))
         self.connection.close()
