@@ -17,7 +17,7 @@ class ValidBasicAuth(BasicAuth):
     def check_auth(self, username, password, allowed_roles, resource, method):
         self.set_request_auth_value(self.request_auth_value)
         return username == 'admin' and password == 'secret' and  \
-            (allowed_roles == ['admin'] if allowed_roles else True)
+            ('admin' in allowed_roles if allowed_roles else True)
 
 
 class BadBasicAuth(BasicAuth):
@@ -26,7 +26,7 @@ class BadBasicAuth(BasicAuth):
 
 class ValidTokenAuth(TokenAuth):
     def check_auth(self, token, allowed_roles, resource, method):
-        return token == 'test_token' and (allowed_roles == ['admin'] if
+        return token == 'test_token' and ('admin' in allowed_roles if
                                           allowed_roles else True)
 
 
@@ -34,7 +34,7 @@ class ValidHMACAuth(HMACAuth):
     def check_auth(self, userid, hmac_hash, headers, data, allowed_roles,
                    resource, method):
         return userid == 'admin' and hmac_hash == 'secret' and  \
-            (allowed_roles == ['admin'] if allowed_roles else True)
+            ('admin' in allowed_roles if allowed_roles else True)
 
 
 class BadHMACAuth(HMACAuth):
@@ -52,10 +52,16 @@ class TestBasicAuth(TestBase):
                            self.content_type]
         self.invalid_auth = [('Authorization', 'Basic IDontThinkSo'),
                              self.content_type]
+        self.setUpRoles()
+        self.app.set_defaults()
+
+    def setUpRoles(self):
         for _, schema in self.app.config['DOMAIN'].items():
             schema['allowed_roles'] = ['admin']
+            schema['allowed_read_roles'] = ['reader']
             schema['allowed_item_roles'] = ['admin']
-        self.app.set_defaults()
+            schema['allowed_item_read_roles'] = ['reader']
+            schema['allowed_item_write_roles'] = ['editor']
 
     def test_custom_auth(self):
         self.assertTrue(isinstance(self.app.auth, ValidBasicAuth))
@@ -91,10 +97,10 @@ class TestBasicAuth(TestBase):
         r = self.test_client.post(self.known_resource_url,
                                   data=json.dumps({"k": "value"}),
                                   headers=self.valid_auth)
-        self.assert400(r.status_code)
+        self.assertValidationErrorStatus(r.status_code)
         r = self.test_client.delete(self.known_resource_url,
                                     headers=self.valid_auth)
-        self.assert200(r.status_code)
+        self.assert204(r.status_code)
 
     def test_authorized_item_access(self):
         r = self.test_client.get(self.item_id_url, headers=self.valid_auth)
@@ -144,6 +150,7 @@ class TestBasicAuth(TestBase):
             del(settings['public_methods'])
         self.app.set_defaults()
         del(domain['peopleinvoices'])
+        del(domain['internal_transactions'])
         for resource in domain:
             url = self.app.config['URLS'][resource]
             r = self.test_client.get(url)
@@ -216,6 +223,22 @@ class TestBasicAuth(TestBase):
         self.assertTrue(('WWW-Authenticate', 'Basic realm:"%s"' %
                          eve.__package__) in r.headers.to_wsgi_list())
 
+    def test_allowed_roles_does_not_change(self):
+        self.test_client.get(self.known_resource_url)
+        resource = self.app.config['DOMAIN'][self.known_resource]
+        self.assertEqual(resource['allowed_roles'], ['admin'])
+
+    def test_allowed_item_roles_does_not_change(self):
+        self.test_client.get(self.item_id_url)
+        resource = self.app.config['DOMAIN'][self.known_resource]
+        self.assertEqual(resource['allowed_item_roles'], ['admin'])
+
+    def test_ALLOWED_ROLES_does_not_change(self):
+        self.app.config['ALLOWED_ROLES'] = ['admin']
+        self.app.config['ALLOWED_READ_ROLES'] = ['reader']
+        self.test_client.get('/')
+        self.assertEqual(self.app.config['ALLOWED_ROLES'], ['admin'])
+
 
 class TestTokenAuth(TestBasicAuth):
     def setUp(self):
@@ -224,6 +247,7 @@ class TestTokenAuth(TestBasicAuth):
         self.test_client = self.app.test_client()
         self.valid_auth = [('Authorization', 'Basic dGVzdF90b2tlbjo='),
                            self.content_type]
+        self.setUpRoles()
 
     def test_custom_auth(self):
         self.assertTrue(isinstance(self.app.auth, ValidTokenAuth))
@@ -236,6 +260,7 @@ class TestHMACAuth(TestBasicAuth):
         self.test_client = self.app.test_client()
         self.valid_auth = [('Authorization', 'admin:secret'),
                            self.content_type]
+        self.setUpRoles()
 
     def test_custom_auth(self):
         self.assertTrue(isinstance(self.app.auth, ValidHMACAuth))
@@ -456,6 +481,17 @@ class TestUserRestrictedAccess(TestBase):
         self.assertEqual(data['_items'][0]['ref'],
                          json.loads(self.data)['ref'])
 
+    def test_post_bandwidth_saver_off_resource_auth(self):
+        """ Test that when BANDWIDTH_SAVER is turned off the auth_field is
+        not exposed in the response payload
+        """
+        self.app.config['BANDWIDTH_SAVER'] = False
+        r = self.app.test_client().post(self.url, data=self.data,
+                                        headers=self.valid_auth,
+                                        content_type='application/json')
+        r, status = self.parse_response(r)
+        self.assertTrue('username' not in r)
+
     def test_put(self):
         new_ref = "9999999999999999999999999"
         changes = json.dumps({"ref": new_ref})
@@ -519,6 +555,28 @@ class TestUserRestrictedAccess(TestBase):
         self.assert200(status)
         self.assertEqual(data['ref'], new_ref)
 
+    def test_put_bandwidth_saver_off_resource_auth(self):
+        """ Test that when BANDWIDTH_SAVER is turned off the auth_field is
+        not exposed in the response payload
+        """
+        self.app.config['BANDWIDTH_SAVER'] = False
+
+        new_ref = "9999999999999999999999999"
+        changes = json.dumps({"ref": new_ref})
+
+        # post document
+        data, status = self.post()
+
+        url = '%s/%s' % (self.url, data['_id'])
+
+        # perform put
+        headers = [('If-Match', data['_etag']), self.valid_auth[0]]
+        response, status = self.parse_response(
+            self.test_client.put(url, data=json.dumps(changes),
+                                 headers=headers,
+                                 content_type='application/json'))
+        self.assertTrue('username' not in response)
+
     def test_patch(self):
         new_ref = "9999999999999999999999999"
         changes = json.dumps({"ref": new_ref})
@@ -557,7 +615,7 @@ class TestUserRestrictedAccess(TestBase):
         # delete the document we just inserted
         response, status = self.parse_response(
             self.test_client.delete(self.url, headers=self.valid_auth))
-        self.assert200(status)
+        self.assert204(status)
 
         # we now get an empty items list (other documents in collection are
         # filtered by auth).
@@ -590,7 +648,7 @@ class TestUserRestrictedAccess(TestBase):
         # delete the document
         response, status = self.parse_response(
             self.test_client.delete(url, headers=headers))
-        self.assert200(status)
+        self.assert204(status)
 
         # make sure no other document has been deleted.
         cursor = _db.contacts.find()
