@@ -7,6 +7,7 @@ from eve.tests.test_settings import MONGO_DBNAME
 from eve import STATUS_OK, LAST_UPDATED, ID_FIELD, DATE_CREATED, ISSUES, \
     STATUS, ETAG
 from eve.methods.post import post
+from eve.methods.post import post_internal
 
 
 class TestPost(TestBase):
@@ -24,13 +25,17 @@ class TestPost(TestBase):
 
     def test_validation_error(self):
         r, status = self.post(self.known_resource_url, data={"ref": "123"})
-        self.assert400(status)
+        self.assertValidationErrorStatus(status)
         self.assertValidationError(r, {'ref': 'min length is 25'})
 
         r, status = self.post(self.known_resource_url, data={"prog": 123})
-        self.assert400(status)
+        self.assertValidationErrorStatus(status)
         self.assertValidationError(r, {'ref': 'required'})
 
+    def test_post_empty_bulk_insert(self):
+        r, status = self.post(self.empty_resource_url, data=[])
+        self.assert400(status)
+
     def test_post_empty_resource(self):
         data = []
         for _ in range(10):
@@ -76,6 +81,20 @@ class TestPost(TestBase):
         data = {test_field: test_value}
         self.assertPostItem(data, test_field, test_value)
 
+    def test_post_integer_zero(self):
+        del(self.domain['contacts']['schema']['ref']['required'])
+        test_field = "aninteger"
+        test_value = 0
+        data = {test_field: test_value}
+        self.assertPostItem(data, test_field, test_value)
+
+    def test_post_float_zero(self):
+        del(self.domain['contacts']['schema']['ref']['required'])
+        test_field = "afloat"
+        test_value = 0.0
+        data = {test_field: test_value}
+        self.assertPostItem(data, test_field, test_value)
+
     def test_post_dict(self):
         del(self.domain['contacts']['schema']['ref']['required'])
         test_field = "location"
@@ -132,16 +151,35 @@ class TestPost(TestBase):
         data = {"ref": "QQQQQQQQQQQQQQQQQQQQQQQQQ"}
         self.assertPostItem(data, 'title', False)
 
-    def test_multi_post(self):
+    def test_multi_post_valid(self):
         data = [
             {"ref": "9234567890123456789054321"},
-            {"prog": 7},
+            {"ref": "5432112345678901234567890", "role": ["agent"]},
+        ]
+        r, status = self.post(self.known_resource_url, data=data)
+        self.assert201(status)
+        results = r['_items']
+
+        self.assertEqual(results[0]['_status'], 'OK')
+        self.assertEqual(results[1]['_status'], 'OK')
+
+        with self.app.test_request_context():
+            contacts = self.app.data.driver.db['contacts']
+            r = contacts.find({"ref": "9234567890123456789054321"}).count()
+            self.assertTrue(r == 1)
+            r = contacts.find({"ref": "5432112345678901234567890"}).count()
+            self.assertTrue(r == 1)
+
+    def test_multi_post_invalid(self):
+        data = [
+            {"ref": "9234567890123456789054321"},
+            {"prog": 9999},
             {"ref": "5432112345678901234567890", "role": ["agent"]},
             {"ref": self.item_ref},
             {"ref": "9234567890123456789054321", "tid": "12345678"},
         ]
         r, status = self.post(self.known_resource_url, data=data)
-        self.assert400(status)
+        self.assertValidationErrorStatus(status)
         results = r['_items']
 
         self.assertEqual(results[0]['_status'], 'OK')
@@ -152,17 +190,16 @@ class TestPost(TestBase):
         self.assertValidationError(results[4], {'tid': 'ObjectId'})
 
         self.assertTrue(ID_FIELD not in results[0])
+        self.assertTrue(ID_FIELD not in results[1])
         self.assertTrue(ID_FIELD not in results[2])
+        self.assertTrue(ID_FIELD not in results[3])
 
-        # items on which validation failed should not be inserted into the db
-        _, status = self.get(self.known_resource_url, 'where=prog==7')
-        self.assert404(status)
-
-        # valid items part of a request containing invalid document should not
-        # be inserted into the db
-        _, status = self.get(self.known_resource_url,
-                             'where=ref==9234567890123456789054321')
-        self.assert404(status)
+        with self.app.test_request_context():
+            contacts = self.app.data.driver.db['contacts']
+            r = contacts.find({"prog": 9999}).count()
+            self.assertTrue(r == 0)
+            r = contacts.find({"ref": "9234567890123456789054321"}).count()
+            self.assertTrue(r == 0)
 
     def test_post_x_www_form_urlencoded(self):
         test_field = "ref"
@@ -177,7 +214,7 @@ class TestPost(TestBase):
     def test_post_referential_integrity(self):
         data = {"person": self.unknown_item_id}
         r, status = self.post('/invoices/', data=data)
-        self.assert400(status)
+        self.assertValidationErrorStatus(status)
         expected = ("value '%s' must exist in resource '%s', field '%s'" %
                     (self.unknown_item_id, 'contacts',
                      self.app.config['ID_FIELD']))
@@ -188,17 +225,46 @@ class TestPost(TestBase):
         self.assert201(status)
         self.assertPostResponse(r)
 
+    def test_post_referential_integrity_list(self):
+        data = {"invoicing_contacts": [self.item_id, self.unknown_item_id]}
+        r, status = self.post('/invoices/', data=data)
+        self.assertValidationErrorStatus(status)
+        expected = ("value '%s' must exist in resource '%s', field '%s'" %
+                    (self.unknown_item_id, 'contacts',
+                     self.app.config['ID_FIELD']))
+        self.assertValidationError(r, {'invoicing_contacts': expected})
+
+        data = {"invoicing_contacts": [self.item_id, self.item_id]}
+        r, status = self.post('/invoices/', data=data)
+        self.assert201(status)
+        self.assertPostResponse(r)
+
     def test_post_allow_unknown(self):
         del(self.domain['contacts']['schema']['ref']['required'])
         data = {"unknown": "unknown"}
         r, status = self.post(self.known_resource_url, data=data)
-        self.assert400(status)
+        self.assertValidationErrorStatus(status)
         self.assertValidationError(r, {'unknown': 'unknown'})
-        self.app.config['DOMAIN'][self.known_resource]['allow_unknown'] = True
+
+        # since resource settings are only set at app startup we set
+        # those that influence the 'allow_unknown' property by hand (so we
+        # don't have to re-initialize the whole app.)
+        settings = self.app.config['DOMAIN'][self.known_resource]
+        settings['allow_unknown'] = True
+        settings['datasource']['projection'] = None
+
         r, status = self.post(self.known_resource_url, data=data)
         self.assert201(status)
         self.assertPostResponse(r)
 
+        # test that the unknown field is also returned with subsequent get
+        # requests
+        id = r[self.app.config['ID_FIELD']]
+        r = self.test_client.get('%s/%s' % (self.known_resource_url, id))
+        r_data = json.loads(r.get_data())
+        self.assertTrue('unknown' in r_data)
+        self.assertEqual('unknown', r_data['unknown'])
+
     def test_post_with_content_type_charset(self):
         test_field = 'ref'
         test_value = "1234567890123456789054321"
@@ -285,13 +351,13 @@ class TestPost(TestBase):
     def test_custom_issues(self):
         self.app.config['ISSUES'] = 'errors'
         r, status = self.post(self.known_resource_url, data={"ref": "123"})
-        self.assert400(status)
+        self.assertValidationErrorStatus(status)
         self.assertTrue('errors' in r and ISSUES not in r)
 
     def test_custom_status(self):
         self.app.config['STATUS'] = 'report'
         r, status = self.post(self.known_resource_url, data={"ref": "123"})
-        self.assert400(status)
+        self.assertValidationErrorStatus(status)
         self.assertTrue('report' in r and STATUS not in r)
 
     def test_custom_etag_update_date(self):
@@ -399,11 +465,47 @@ class TestPost(TestBase):
         test_value = 'a random value'
         data = {test_field: test_value}
         r, status = self.post(self.known_resource_url, data=data)
-        self.assert400(status)
-        # this will pass as value matches 'default' setting.
+        self.assertValidationErrorStatus(status)
+        # this will not pass even if value matches 'default' setting.
+        # (hey it's still a read-onlu field so you can't reset it)
         test_value = 'default'
         data = {test_field: test_value}
-        self.assertPostItem(data, test_field, test_value)
+        r, status = self.post(self.known_resource_url, data=data)
+        self.assertValidationErrorStatus(status)
+
+    def test_post_readonly_in_dict(self):
+        # Test that a post with a readonly field inside a dict is properly
+        # validated (even if it has a defult value)
+        del(self.domain['contacts']['schema']['ref']['required'])
+        test_field = 'dict_with_read_only'
+        test_value = {'read_only_in_dict': 'default'}
+        data = {test_field: test_value}
+        r, status = self.post(self.known_resource_url, data=data)
+        self.assertValidationErrorStatus(status)
+
+    def test_post_keyschema_dict(self):
+        """ make sure Cerberus#48 is fixed """
+        del(self.domain['contacts']['schema']['ref']['required'])
+        r, status = self.post(self.known_resource_url,
+                              data={"keyschema_dict": {"k1": "1"}})
+        self.assertValidationErrorStatus(status)
+        issues = r[ISSUES]
+        self.assertTrue('keyschema_dict' in issues)
+        self.assertEqual(issues['keyschema_dict'],
+                         {'k1': 'must be of integer type'})
+
+        r, status = self.post(self.known_resource_url,
+                              data={"keyschema_dict": {"k1": 1}})
+        self.assert201(status)
+
+    def test_post_internal(self):
+        # test that post_internal is available and working properly.
+        test_field = 'ref'
+        test_value = "1234567890123456789054321"
+        payload = {test_field: test_value}
+        with self.app.test_request_context(self.known_resource_url):
+            r, _, _, status = post_internal(self.known_resource, payl=payload)
+        self.assert201(status)
 
     def perform_post(self, data, valid_items=[0]):
         r, status = self.post(self.known_resource_url, data=data)
