@@ -7,8 +7,6 @@ MONGO_PASSWORD = 'test_pw'
 MONGO_DBNAME = 'eve_test'
 ID_FIELD = '_id'
 
-SERVER_NAME = 'localhost:5000'
-
 RESOURCE_METHODS = ['GET', 'POST', 'DELETE']
 ITEM_METHODS = ['GET', 'PATCH', 'DELETE', 'PUT']
 ITEM_CACHE_CONTROL = ''
@@ -99,7 +97,30 @@ contacts = {
             'type': 'string',
             'default': 'default',
             'readonly': True
-        }
+        },
+        'dict_with_read_only': {
+            'type': 'dict',
+            'schema': {
+                'read_only_in_dict': {
+                    'type': 'string',
+                    'default': 'default',
+                    'readonly': True
+                }
+            }
+        },
+        'key1': {
+            'type': 'string',
+        },
+        'keyschema_dict': {
+            'type': 'dict',
+            'keyschema': {'type': 'integer'}
+        },
+        'aninteger': {
+            'type': 'integer',
+        },
+        'afloat': {
+            'type': 'float',
+        },
     }
 }
 
@@ -120,6 +141,37 @@ invoices = {
         'person': {
             'type': 'objectid',
             'data_relation': {'resource': 'contacts'}
+        },
+        'invoicing_contacts': {
+            'type': 'list',
+            'data_relation': {'resource': 'contacts'}
+        }
+    }
+}
+
+# This resource is used to test app initialization when using resource
+# level versioning
+versioned_invoices = copy.deepcopy(invoices)
+versioned_invoices['versioning'] = True
+
+companies = {
+    'item_title': 'company',
+    'schema': {
+        'departments': {
+            'type': 'list',
+            'schema': {
+                'type': 'dict',
+                'schema': {
+                    'title': {'type': 'string'},
+                    'members': {
+                        'type': 'list',
+                        'schema': {
+                            'type': 'objectid',
+                            'data_relation': {'resource': 'contacts'}
+                        }
+                    }
+                }
+            }
         }
     }
 }
@@ -143,13 +195,22 @@ users_invoices = copy.deepcopy(invoices)
 users_invoices['url'] = 'users/<regex("[a-f0-9]{24}"):person>/invoices'
 users_invoices['datasource'] = {'source': 'invoices'}
 
+internal_transactions = {
+    'resource_methods': ['GET'],
+    'item_methods': ['GET'],
+    'internal_resource': True
+}
+
 DOMAIN = {
     'contacts': contacts,
     'users': users,
     'users_overseas': users_overseas,
     'invoices': invoices,
+    'versioned_invoices': versioned_invoices,
     'payments': payments,
     'empty': empty,
     'restricted': user_restricted_access,
     'peopleinvoices': users_invoices,
+    'companies': companies,
+    'internal_transactions': internal_transactions,
 }
