@@ -6,12 +6,13 @@ from cerberus import errors
 from eve.auth import BasicAuth, TokenAuth, HMACAuth
 from hashlib import sha1
 import hmac
-from flask import request
+from flask import request, make_response
 from redis import Redis
+from werkzeug.routing import BaseConverter
+from uuid import UUID
 
 redis = Redis()
 
-
 class UUIDValidator(Validator):
     """
     Extends the base mongo validator adding support for the uuid data-type
@@ -23,8 +24,6 @@ class UUIDValidator(Validator):
             self._error(field, "value '%s' cannot be converted to a UUID" %
                         value)
 
-from werkzeug.routing import BaseConverter
-from uuid import UUID
 
 
 class UUIDConverter(BaseConverter):
@@ -64,6 +63,22 @@ class UUIDEncoder(BaseJSONEncoder):
             # will properly render ObjectIds, datetimes, etc.)
             return super(UUIDEncoder, self).default(obj)
 
+class HMACAuth(HMACAuth):
+    def check_auth(self, userid, hmac_hash, headers, data, allowed_roles,
+                   resource, method):
+        # use Eve's own db driver; no additional connections/resources are
+        # used
+        return True
+        accounts = app.data.driver.db['accounts']
+        user = accounts.find_one({'userid': userid})
+        if user:
+            secret_key = user['secret_key']
+        # in this implementation we only hash request data, ignoring the
+        # headers.
+        return user and \
+            hmac.new(str(secret_key), str(data), sha1).hexdigest() == \
+                hmac_hash
+
 
 class TAuth(TokenAuth):
     def check_auth(self, token, allowed_roles, resource, method):
@@ -72,6 +87,7 @@ class TAuth(TokenAuth):
 class Auth(BasicAuth):
     def check_auth(self, username, password, allowed_roles, resource, method):
         #print "username", username
+        import ipdb; ipdb.set_trace()  # XXX BREAKPOINT
         #self.request_auth_value = username
         self.set_request_auth_value(username)
         return username == 'token'
@@ -92,8 +108,9 @@ def getting_contatti(request, lookup):
     #lookup["_id"] = "52c27a8338345b482a60c3a4"
     lookup["username"] = {'$exists': False}
 
-def pre_post(resource, request):
+def pre_post(resource, request, lookup):
     print resource, request
+    print lookup
     #lookup["ciao"] = "ciao"
     #del(documents[0]['etag'])
 
@@ -105,7 +122,6 @@ def gotit(resource, request, response=None):
 
     print ('get', resource)
     print (request)
-    print (response)
 
 def fetched_contacts(res, items):
     import pdb; pdb.set_trace()  # XXX BREAKPOINT
@@ -148,7 +164,15 @@ def fetching(resource, document):
 def fetching_item(resource, document):
     document['number'] = 1
 
+def edit_request(resource, r):
+    r.max_results = 1
+
 app = Eve(validator=Validator, redis=redis)
+
+#@app.errorhandler(401)
+#def handle_sqlalchemy_assertion_error(err):
+#    return make_response("hello", 401)
+
 #app = Eve(json_encoder=UUIDEncoder, validator=UUIDValidator)
 
 #@app.route('/test')
@@ -165,9 +189,11 @@ app = Eve(validator=Validator, redis=redis)
 #app.on_fetched_resource += fetched_contacts
 #app.on_fetched_item += fetched_contact
 #app.on_insert_contacts += posting
-app.on_insert += before_insert
+#app.on_insert += before_insert
 #app.on_getting_contacts += getting_contatti
 #app.on_post_DELETE += resource_post
+#app.on_post_GET += gotit
+#app.on_parsed_request += edit_request
 
 if __name__ == '__main__':
     #app.events.on_getting += pre
